from matplotlib import cm, ticker
import numpy as np
import os, glob, stats
import matplotlib
import copyFile
showPlots = True
try:
  os.environ["DISPLAY"] #Detects if display is available
except KeyError:
  showPlots = False
  matplotlib.use('Agg') #Need to set this so doesn't try (and fail) to open interactive graphics window
import matplotlib.pyplot as plt
matplotlib.rc('font', size=7)
matplotlib.rc('axes', titlesize=7)
colors = ['#6495ED','r','#6da81b','#483D8B','#FF8C00','#2E8B57','#800080','#008B8B','#1E90FF']

#-------
#Inputs
#-------

#copy data files to current dir
copy = False
mainDir = '/home/mnguyen/CHE210D/whitmer/PA-PB/'
filetoCopy = ['chargesMC.txt','chargesMD.txt','ionization.txt','PEMC.txt','Pacc.txt','trajectoryMD.dcd','rg.txt']
#calculate means of observable, if False read means and error of observable from data file
extract = False
data = 'data.txt'

#simulation parameters
ks = [0.1,0.2,1,3,6]
muPAs =[-8,-4,0,4,8]
muPBs = [-8,-4,0,4,8]
Ns = [60]
NP = 2 #number of polymer chains

#parameters to for rg calculation
stride = 1
frames = 100000 
top = 'trajectory_init.pdb'
#---------------------------

#copy files to current directory
if copy:
    copyFile.copy(muPAs,muPBs,ks,Ns,filetoCopy,mainDir)

#creating dictionary of data files
#dictionary, key=[k,N,muPA,muPB], value = [ionization.txt,rg.txt,PEMC.txt,Pacc.txt]
fileDict = {}
for N in Ns:
    for k in ks:
        for muPA in muPAs:
            for muPB in muPBs:
                key = (k,N,muPA,muPB)
                fHeader = 'k{k}_N{N}_m{muPA}{muPB}'.format(k=k,N=N, muPA=muPA, muPB=muPB)
                value = [fHeader + '_ionization.txt',fHeader + '_rg.txt', fHeader + '_PEMC.txt', fHeader + '_Pacc.txt']
                fileDict.update({key:value})

###read file and extract data and append to dictionary of same keys, value = [[fA, err],[fB,err],[PE,err],[RgPA, std], [RgPB, std]]

obsDict = {}
if extract:
    print('calculating means of fA, fB')
    #ionization
    for key in fileDict.keys():
        file = fileDict[key][0]
        if os.path.isfile(file):
            obsDict.update({key:[]})
            cols = [1,2]
            f = open(file,'r')
            for col in cols:
                #do stats
                mean,std = stats.getStat(f,col)
                obsDict[key].append([mean,std])
    #PE
    print('calculating means of PE')
    for key in obsDict.keys():
        file = fileDict[key][2]
        if os.path.isfile(file):
            col = 1
            f = open(file,'r')
            mean,std = stats.getStat(f,col)
            obsDict[key].append([mean,std])
    #rg
    print('calculating means of Rgs')
    for key in sorted(obsDict.keys()):
        file = fileDict[key][1]
        if os.path.isfile(file):
            cols = [1,2]
            f = open(file,'r')
            for col in cols:
                #do stats
                mean,std = stats.getStat(f,col)
                obsDict[key].append([mean,std])
        else:
            obsDict[key].append([[0],[0]])
            obsDict[key].append([[0],[0]])

    #write to text file
    f = open('data.txt','w')
    f.write('# k\tN\tmuPA\tmuPB\tfA\tfAerr\tfB\tfBerr\tPE\t\tPEerr\tRgA\tRgAerr\tRgB\tRgBerr\n')
    for key in sorted(obsDict.keys()):
        k,N,muPA,muPB = key
        fA = obsDict[key][0][0]
        fAerr = obsDict[key][0][1]
        fB = obsDict[key][1][0]
        fBerr = obsDict[key][1][1]
        PE = obsDict[key][2][0]
        PEerr = obsDict[key][2][1]
        RgA = obsDict[key][3][0]
        RgAerr = obsDict[key][3][1]
        RgB = obsDict[key][4][0]
        RgBerr = obsDict[key][4][1]
        s = '%2.1f\t%i\t%i\t%i\t%2.4f\t%2.4f\t%2.4f\t%2.4f\t%5.4f\t%5.4f\t%5.4f\t%5.4f\t%5.4f\t%5.4f\n' %(k,N,muPA,muPB,fA,fAerr,fB,fBerr,PE,PEerr,RgA,RgAerr,RgB,RgBerr)
        f.write(s)
    f.close()     
else:
    print('reading {}'.format(data))
    f = open(data,'r')
    lines = f.readlines()
    for line in lines:
        if line.startswith('#'):
            obs = line.split()[1:]
            kId = obs.index('k')
            NId = obs.index('N')
            muPAId = obs.index('muPA')
            muPBId = obs.index('muPB')
            fAId = obs.index('fA')
            fAerrId = obs.index('fAerr')
            fBId = obs.index('fB')
            fBerrId = obs.index('fBerr')
            PEId = obs.index('PE')
            PEerrId = obs.index('PEerr')
            RgAId = obs.index('RgA')
            RgAerrId = obs.index('RgAerr')
            RgBId = obs.index('RgB')
            RgBerrId = obs.index('RgBerr')
        else:
            cols = line.split()
            key = (cols[kId],cols[NId],cols[muPAId],cols[muPBId])
            key = tuple([float(val) for val in key])
            value = [[cols[fAId],cols[fAerrId]],[cols[fBId],cols[fBerrId]],[cols[PEId],cols[PEerrId]],[cols[RgAId],cols[RgAerrId]],[cols[RgBId],cols[RgBerrId]]]
            value = [[float(a),float(b)] for [a,b] in value]
            obsDict.update({key:value})

#getting matrices of observables as a function of muPA and muPB
fADict = {} #key = k, value = array
fBDict = {}
PEDict = {}
RgADict = {}
RgBDict = {}
N=60
dicts = [fADict,fBDict,PEDict,RgADict,RgBDict]
obsshort = ['fA','fB','PE','RgA','RgB']
obsToExtract = ['fA','fB','PE','RgA','RgB']
keys = sorted(obsDict.keys())
for id,dict in enumerate(dicts):
    for k in ks:
        array = -10*np.ones([len(muPAs),len(muPBs)])
        for i,muPA in enumerate(muPAs):
            for j,muPB in enumerate(muPBs):
                key = (k,N,muPA,muPB)
                key = tuple([float(val) for val in key])
                obsId = obsshort.index(obsToExtract[id])
                if key in keys:
                    val = obsDict[key][obsId][0]
                else: 
                    val = -1000
                array[i,j] = val
        dict.update({k:array})

print('plotting')
#PLOT
interpolation = 'spline16'

#heat maps of fA vs muPA and muPB
for k in ks:
    plt.figure(figsize=[2,2])
    im=plt.imshow(fADict[k],cmap=plt.cm.get_cmap('jet'),alpha = 1,interpolation=interpolation,extent = [min(muPBs),max(muPBs),min(muPAs),max(muPAs)],origin='lower')
    title = 'fAmap_k{}'.format(k)
    plt.colorbar(im,label='$f_A$')
    plt.xlabel('$\mu_{PB}$')
    plt.ylabel('$\mu_{PA}$')
    plt.title(title,loc='center')
    ax = plt.gca()
    ax.xaxis.set_major_locator(ticker.MultipleLocator(2.))
    ax.yaxis.set_major_locator(ticker.MultipleLocator(2.))
    plt.savefig(title+'.png',dpi=500,bbox_inches='tight')
#plt.show()

#heat maps of fB vs muPA and muPB
for k in ks:
    plt.figure(figsize=[2,2])
    im=plt.imshow(fBDict[k],cmap=plt.cm.get_cmap('jet'),alpha = 1,interpolation=interpolation,extent = [min(muPBs),max(muPBs),min(muPAs),max(muPAs)],origin='lower')
    plt.colorbar(im,label='$f_B$')
    title = 'fBmap_k{}'.format(k)
    plt.xlabel('$\mu_{PB}$')
    plt.ylabel('$\mu_{PA}$')
    plt.title(title,loc='center')
    ax = plt.gca()
    ax.xaxis.set_major_locator(ticker.MultipleLocator(2.))
    ax.yaxis.set_major_locator(ticker.MultipleLocator(2.))
    plt.savefig(title+'.png',dpi=500,bbox_inches='tight')
#plt.show()

#heat maps of PE vs muPA and muPB
PEmin = 50
PEmax = 0
for k in ks:
    if np.min(PEDict[k])<PEmin:
        PEmin = np.min(PEDict[k])
    if np.max(PEDict[k])>PEmax:
        PEmax = np.max(PEDict[k])
for k in ks:
    plt.figure(figsize=[2,2])
    im=plt.imshow(PEDict[k],cmap=plt.cm.get_cmap('jet'),alpha = 1,interpolation=interpolation,extent = [min(muPBs),max(muPBs),min(muPAs),max(muPAs)],origin='lower')
    plt.clim(PEmin,PEmax)
    plt.colorbar(im,label='$PE$')
    plt.xlabel('$\mu_{PB}$')
    plt.ylabel('$\mu_{PA}$')
    ax = plt.gca()
    ax.xaxis.set_major_locator(ticker.MultipleLocator(2.))
    ax.yaxis.set_major_locator(ticker.MultipleLocator(2.))
    title = 'PEmap_k{}'.format(k)
    plt.title(title,loc='center')
    plt.savefig(title+'.png',dpi=500,bbox_inches='tight')
#plt.show()

#heat maps of RgA vs muPA and muPB
Rgmin = 50
Rgmax = 0
for k in ks:
    if np.min(RgADict[k])<Rgmin:
        Rgmin = np.min(RgADict[k])
    if np.max(RgADict[k])>Rgmax:
        Rgmax = np.max(RgADict[k])

for k in ks:
    plt.figure(figsize=[2,2])
    im=plt.imshow(RgADict[k],cmap=plt.cm.get_cmap('jet'),alpha = 1,interpolation=interpolation,extent = [min(muPBs),max(muPBs),min(muPAs),max(muPAs)],origin='lower')
    plt.clim(Rgmin,Rgmax)
    plt.colorbar(im,label='$Rg_A$')
    plt.xlabel('$\mu_{PB}$')
    plt.ylabel('$\mu_{PA}$')
    ax = plt.gca()
    ax.xaxis.set_major_locator(ticker.MultipleLocator(2.))
    ax.yaxis.set_major_locator(ticker.MultipleLocator(2.))
    title = 'RgAmap_k{}'.format(k)
    plt.title(title,loc='center')
    plt.savefig(title+'.png',dpi=500,bbox_inches='tight')
#plt.show()

#heat maps of RgB vs muPA and muPB
Rgmin = 50
Rgmax = 0
for k in ks:
    if np.min(RgBDict[k])<Rgmin:
        Rgmin = np.min(RgBDict[k])
    if np.max(RgBDict[k])>Rgmax:
        Rgmax = np.max(RgBDict[k])

for k in ks:
    plt.figure(figsize=[2,2])
    im=plt.imshow(RgBDict[k],cmap=plt.cm.get_cmap('jet'),alpha = 1,interpolation=interpolation,extent = [min(muPBs),max(muPBs),min(muPAs),max(muPAs)],origin='lower')
    plt.clim(Rgmin,Rgmax)
    plt.colorbar(im,label='$Rg_A$')
    plt.xlabel('$\mu_{PB}$')
    plt.ylabel('$\mu_{PA}$')
    ax = plt.gca()
    ax.xaxis.set_major_locator(ticker.MultipleLocator(2.))
    ax.yaxis.set_major_locator(ticker.MultipleLocator(2.))
    title = 'RgBmap_k{}'.format(k)
    plt.title(title,loc='center')
    plt.savefig(title+'.png',dpi=500,bbox_inches='tight')

#fA vs kappa
N = 60
for i,muPA in enumerate(muPAs):
    fig,ax = plt.subplots(nrows=1, ncols=1, figsize=[2,2])
    ax.set_prop_cycle('color', colors)
    for j,muPB in enumerate(muPBs):
        yerr = []
        y = []
        for k in ks:
            key = (k,N,muPA,muPB)
            y.append(obsDict[key][0][0]) 
            yerr.append(obsDict[key][0][1])
        ax.errorbar(ks, y, yerr=yerr,marker='o',ls='-',lw=0.75,mfc="None",ms=2, capsize=2,label = '$\mu_{PA},\mu_{PB}$'+ ' = {},{}'.format(muPA,muPB))  
    ax.set_xscale("log")
    ax.legend(loc='best',prop={'size': 5})
    plt.xlabel('$\kappa$')
    plt.ylabel('$f_A$')
    plt.ylim(0,1)
    title = 'fAvsK_muPA{}'.format(muPA)
    plt.title(title,loc='center')
    plt.savefig(title+'.png',dpi=500,bbox_inches='tight')
#fB vs kappa
N = 60
for i,muPB in enumerate(muPBs):
    fig,ax = plt.subplots(nrows=1, ncols=1, figsize=[2,2])
    ax.set_prop_cycle('color', colors)
    for j,muPA in enumerate(muPAs):
        yerr = []
        y = []
        for k in ks:
            key = (k,N,muPA,muPB)
            y.append(obsDict[key][1][0])
            yerr.append(obsDict[key][1][1])
        ax.errorbar(ks, y, yerr=yerr,marker='o',ls='-',lw=0.75,mfc="None",ms=2, capsize=2,label = '$\mu_{PA},\mu_{PB}$'+ ' = {},{}'.format(muPA,muPB))
    ax.set_xscale("log")
    ax.legend(loc='best',prop={'size': 5})
    plt.ylim(0,1)
    plt.xlabel('$\kappa$')
    plt.ylabel('$f_B$')
    title = 'fBvsK_muPB{}'.format(muPB)
    plt.title(title,loc='center')
    plt.savefig(title+'.png',dpi=500,bbox_inches='tight')
#PE vs kappa
N = 60
for i,muPA in enumerate(muPAs):
    fig,ax = plt.subplots(nrows=1, ncols=1, figsize=[3,2])
    ax.set_prop_cycle('color', colors)
    for j,muPB in enumerate(muPBs):
        yerr = []
        y = []
        for k in ks:
            key = (k,N,muPA,muPB)
            y.append(obsDict[key][2][0])
            yerr.append(obsDict[key][2][1])
        ax.errorbar(ks, y, yerr=yerr,marker='o',ls='-',lw=0.75,mfc="None",ms=2, capsize=2,label = '$\mu_{PA},\mu_{PB}$'+ ' = {},{}'.format(muPA,muPB))
    ax.set_xscale("log")
    ax.legend(loc='best',prop={'size': 5})
    plt.xlabel('$\kappa$')
    plt.ylabel('$Potential Energy$')
#    plt.ylim(245,405)
    title = 'PEvsK_muPA{}'.format(muPA)
    plt.title(title,loc='center')
    plt.savefig(title+'.png',dpi=500,bbox_inches='tight')
#plt.show()
